@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2006 The PHP Group                                |
+  | Copyright (c) 1997-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -12,11 +12,11 @@
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
-  | Author: yoya@awm.jp                                                  |
+  | Author: Yoshihiro Yamazaki <yoya@awm.jp>                             |
   +----------------------------------------------------------------------+
 */
 
-/* $Id: header,v 1.16.2.1 2006/01/01 12:50:00 sniper Exp $ */
+/* $Id:$ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -56,15 +56,21 @@ zend_function_entry swfed_functions[] = {
     PHP_ME(swfed,  __construct, NULL, 0)
     PHP_ME(swfed,  input, NULL, 0)
     PHP_ME(swfed,  output, NULL, 0)
+    PHP_ME(swfed,  swfInfo, NULL, 0)
     PHP_ME(swfed,  _destroy_and_exit, NULL, 0) // for debug
+
     PHP_ME(swfed,  getHeaderInfo, NULL, 0)
     PHP_ME(swfed,  setHeaderInfo, NULL, 0)
     PHP_ME(swfed,  getTagList, NULL, 0)
     PHP_ME(swfed,  getTagDetail, NULL, 0)
     PHP_ME(swfed,  getTagData, NULL, 0)
     PHP_ME(swfed,  replaceTagData, NULL, 0)
+    PHP_ME(swfed,  getTagDataByCID, NULL, 0)
+    PHP_ME(swfed,  replaceTagDataByCID, NULL, 0)
     PHP_ME(swfed,  getTagContentsByCID, NULL, 0)
     PHP_ME(swfed,  replaceTagContentsByCID, NULL, 0)
+    PHP_ME(swfed,  removeTag, NULL, 0)
+    PHP_ME(swfed,  printTagData, NULL, 0)
 
     PHP_ME(swfed,  getShapeData, NULL, 0)
     PHP_ME(swfed,  replaceShapeData, NULL, 0)
@@ -89,13 +95,16 @@ zend_function_entry swfed_functions[] = {
     PHP_ME(swfed,  replaceMovieClip, NULL, 0)
 
     PHP_ME(swfed,  setCompressLevel, NULL, 0)
-    PHP_ME(swfed,  swfInfo, NULL, 0)
-//    PHP_ME(swfed,  rebuild, NULL, 0)
+    PHP_ME(swfed,  rebuild, NULL, 0)
+    PHP_ME(swfed,  purgeUselessContents, NULL, 0)
+
+    PHP_ME(swfed,  isShapeTagData, NULL, 0)
+    PHP_ME(swfed,  isBitmapTagData, NULL, 0)
     {NULL, NULL, NULL}	/* Must be the last line in swfed_functions[] */
 };
 /* }}} */
 
-#define SWFED_VERSION "0.35"
+#define SWFED_VERSION "0.40"
 
 /* {{{ swfed_module_entry
  */
@@ -316,6 +325,12 @@ PHP_METHOD(swfed, output) {
     RETURN_STRINGL(new_buff, len, 0);
 }
 
+PHP_METHOD(swfed, swfInfo) {
+    swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
+    swf_object_print(swf);
+    RETURN_TRUE;
+}
+
 PHP_METHOD(swfed, _destroy_and_exit) { // for debug
     swf_object_t *swf = NULL;
     swf = get_swf_object(getThis() TSRMLS_CC);
@@ -401,13 +416,13 @@ PHP_METHOD(swfed, getTagList) {
         RETURN_FALSE; /* XXX */
     }
     swf = get_swf_object(getThis() TSRMLS_CC);
-    tag = swf->tag;
     array_init(return_value);
-    for (tag=swf->tag ; tag ; tag=tag->next) {
+    for (tag=swf->tag_head ; tag ; tag=tag->next) {
         ALLOC_INIT_ZVAL(data);
         array_init(data);
-        add_assoc_long(data, "tag", tag->tag);
-        tag_info = get_swf_tag_info(tag->tag);
+        add_assoc_long(data, "code", tag->code);
+        add_assoc_long(data, "tag", tag->code);
+        tag_info = get_swf_tag_info(tag->code);
         if (tag_info && tag_info->name) {
             add_assoc_string_ex(data,
                                 "tagName", sizeof("tagName"),
@@ -434,7 +449,7 @@ PHP_METHOD(swfed, getTagDetail) {
     }
     swf = get_swf_object(getThis() TSRMLS_CC);
     i = 0;
-    for (tag=swf->tag ; tag ; tag=tag->next) {
+    for (tag=swf->tag_head ; tag ; tag=tag->next) {
         if (i == tag_seqno) {
             break;
         }
@@ -443,14 +458,14 @@ PHP_METHOD(swfed, getTagDetail) {
     if (tag == NULL) {
         RETURN_FALSE;
     }
-    tag_info = get_swf_tag_info(tag->tag);
+    tag_info = get_swf_tag_info(tag->code);
     if ((tag_info == NULL) || (tag_info->detail_handler == NULL)) {
         RETURN_FALSE;
     }
     if (swf_tag_create_input_detail(tag, swf) == NULL) {
         RETURN_FALSE; // can't create detail
     }
-    switch (tag->tag) {
+    switch (tag->code) {
         swf_tag_jpeg_detail_t     *tag_jpeg;
         swf_tag_lossless_detail_t *tag_lossless;
         swf_tag_edit_detail_t     *tag_edit;
@@ -459,6 +474,7 @@ PHP_METHOD(swfed, getTagDetail) {
         swf_tag_sprite_detail_t   *tag_sprite;
         swf_tag_shape_detail_t    *tag_shape;
         swf_tag_place_detail_t    *tag_place;
+	zval *data = NULL;
       case 6:  // DefineBitsJPEG
       case 21: // DefineBitsJPEG2
       case 35: // DefineBitsJPEG3
@@ -498,7 +514,7 @@ PHP_METHOD(swfed, getTagDetail) {
     case 59: // DoInitAction
         tag_action = tag->detail;
         array_init(return_value);
-        if (tag->tag == 59) { // DoInitAction
+        if (tag->code == 59) { // DoInitAction
             add_assoc_long(return_value, "action_sprite", tag_action->action_sprite);
         }
         if (tag_action->action_record && tag_action->action_record_len) {
@@ -536,12 +552,24 @@ PHP_METHOD(swfed, getTagDetail) {
         add_assoc_long(return_value, "fill_styles.count", tag_shape->shape_with_style.styles.fill_styles.count);
         add_assoc_long(return_value, "line_styles.count", tag_shape->shape_with_style.styles.line_styles.count);
 //        tag_shape->shape_with_style.shape_records
+	int *bitmap_id_list, bitmap_id_list_num;
+	bitmap_id_list = swf_tag_shape_bitmap_get_refcid_list(tag, &bitmap_id_list_num);
+	if (bitmap_id_list) {
+	    int i;
+	    ALLOC_INIT_ZVAL(data);
+	    array_init(data);
+	    for (i = 0 ; i < bitmap_id_list_num ; i++) {
+	        add_index_long(data, i , bitmap_id_list[i]);
+	    }
+	    add_assoc_zval(return_value, "bitmap_ref", data);
+	    free(bitmap_id_list);
+	}
         break;
       case 4: // PlaceObject
       case 26: // PlaceObject2
         tag_place = tag->detail;
         array_init(return_value);
-        if ((tag->tag == 4) || tag_place->flag_has_character) {
+        if ((tag->code == 4) || tag_place->flag_has_character) {
             add_assoc_long(return_value, "character_id", tag_place->character_id);
         }
         add_assoc_long(return_value, "depth", tag_place->depth);
@@ -560,19 +588,22 @@ PHP_METHOD(swfed, getTagDetail) {
 PHP_METHOD(swfed, getTagData) {
     long tag_seqno = 0;
     swf_object_t *swf = NULL;
-    unsigned char *data_ref = NULL;
+    unsigned char *data = NULL, *new_buff;
     unsigned long data_len = 0;
     
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &tag_seqno) == FAILURE) {
         RETURN_FALSE;
     }
     swf = get_swf_object(getThis() TSRMLS_CC);
-    data_ref = swf_object_get_tagdata(swf, tag_seqno, &data_len);
-    if (data_ref == NULL) {
+    data = swf_object_get_tagdata(swf, tag_seqno, &data_len);
+    if (data == NULL) {
         fprintf(stderr, "getTagData: Can't get_tagdata\n");
         RETURN_FALSE;
     }
-    RETURN_STRINGL(data_ref, data_len, 1);
+    new_buff = emalloc(data_len);
+    memcpy(new_buff, data, data_len);
+    free(data);
+    RETURN_STRINGL(new_buff, data_len, 0);
 }
 
 PHP_METHOD(swfed, replaceTagData) {
@@ -600,6 +631,49 @@ PHP_METHOD(swfed, replaceTagData) {
     RETURN_TRUE;
 }
 
+PHP_METHOD(swfed, getTagDataByCID) {
+    long cid = 0;
+    swf_object_t *swf = NULL;
+    unsigned char *data_ref = NULL;
+    unsigned long data_len = 0;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &cid) == FAILURE) {
+        RETURN_FALSE;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    data_ref = swf_object_get_tagdata_bycid(swf, cid, &data_len);
+    if (data_ref == NULL) {
+        fprintf(stderr, "getTagDataByCID: Can't get_tagdata_bycid\n");
+        RETURN_FALSE;
+    }
+    RETURN_STRINGL(data_ref, data_len, 1);
+}
+
+PHP_METHOD(swfed, replaceTagDataByCID) {
+    char *data = NULL;
+    unsigned long data_len = 0;
+    long cid = 0;
+    swf_object_t *swf = NULL;
+    int result = 0;
+    switch (ZEND_NUM_ARGS()) {
+      default:
+        WRONG_PARAM_COUNT;
+        RETURN_FALSE; /* XXX */
+      case 2:
+        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &cid, &data, &data_len) == FAILURE) {
+            RETURN_FALSE;
+        }
+        break;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    result = swf_object_replace_tagdata_bycid(swf, cid,
+					      (unsigned char *)data,
+					      data_len);
+    if (result) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;
+}
+
 PHP_METHOD(swfed, getTagContentsByCID) {
     long cid = 0;
     swf_object_t *swf = NULL;
@@ -643,22 +717,73 @@ PHP_METHOD(swfed, replaceTagContentsByCID) {
     RETURN_TRUE;
 }
 
+PHP_METHOD(swfed, removeTag) {
+    long tag_seqno = 0;
+    long tag_seqno_in_sprite = -1;
+    swf_object_t *swf = NULL;
+    int ret;
+
+    switch (ZEND_NUM_ARGS()) {
+      case 1:
+        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &tag_seqno) == FAILURE) {
+            RETURN_FALSE;
+        }
+        break;
+      case 2:
+        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &tag_seqno, &tag_seqno_in_sprite) == FAILURE) {
+            RETURN_FALSE;
+        }
+        break;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    ret = swf_object_remove_tag(swf, tag_seqno, tag_seqno_in_sprite);
+    if (ret) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;
+}
+
+PHP_METHOD(swfed, printTagData) {
+    char *data = NULL;
+    unsigned long data_len = 0;
+    swf_object_t *swf = NULL;
+    int ret = 0;
+    switch (ZEND_NUM_ARGS()) {
+      default:
+        WRONG_PARAM_COUNT;
+        RETURN_FALSE; /* XXX */
+      case 1:
+        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &data, &data_len) == FAILURE) {
+            RETURN_FALSE;
+        }
+        break;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    ret = swf_object_print_tagdata(swf, (unsigned char *)data, data_len);
+    if (ret) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;
+}
 
 PHP_METHOD(swfed, getShapeData) {
     long cid = 0;
     swf_object_t *swf = NULL;
-    unsigned char *data_ref = NULL;
+    unsigned char *data = NULL, *new_buff;
     unsigned long data_len = 0;
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &cid) == FAILURE) {
         RETURN_FALSE;
     }
     swf = get_swf_object(getThis() TSRMLS_CC);
-    data_ref = swf_object_get_shapedata(swf, cid, &data_len);
-    if (data_ref == NULL) {
-        fprintf(stderr, "getShapeData: Can't get_tagcontents_bycid\n");
+    data = swf_object_get_shapedata(swf, cid, &data_len);
+    if (data == NULL) {
+        fprintf(stderr, "getShapeData: Can't swf_object_get_shapedata (cid=%d)\n", cid);
         RETURN_FALSE;
     }
-    RETURN_STRINGL(data_ref, data_len, 1);
+    new_buff = emalloc(data_len);
+    memcpy(new_buff, data, data_len);
+    free(data);
+    RETURN_STRINGL(new_buff, data_len, 0);
 }
 
 PHP_METHOD(swfed, replaceShapeData) {
@@ -700,7 +825,8 @@ PHP_METHOD(swfed, setShapeAdjustMode) {
 }
 
 PHP_METHOD(swfed, getShapeIdListByBitmapRef) {
-    long bitmap_id;
+    int bitmap_id;
+    int *bitmap_id_list, bitmap_id_list_num;
     swf_object_t *swf = NULL;
     swf_tag_t *tag = NULL;
     int i;
@@ -711,12 +837,22 @@ PHP_METHOD(swfed, getShapeIdListByBitmapRef) {
     swf = get_swf_object(getThis() TSRMLS_CC);
     array_init(return_value);
     i = 0;
-    for (tag = swf->tag ; tag ; tag=tag->next) {
-        register int tag_code = tag->tag;
-        if (isShapeTag(tag_code) && (swf_tag_shape_bitmap_get_refcid(tag) == bitmap_id)) {
-            swf_tag_shape_detail_t *swf_tag_shape = tag->detail;
-            add_index_long(return_value, i, (long) swf_tag_shape->shape_id);
-            i++;
+    for (tag = swf->tag_head ; tag ; tag=tag->next) {
+        register int tag_code = tag->code;
+        if (isShapeTag(tag_code)) {
+	    bitmap_id_list = swf_tag_shape_bitmap_get_refcid_list(tag, &bitmap_id_list_num);
+	    if (bitmap_id_list) {
+	        int j;
+		for (j=0 ; j < bitmap_id_list_num ; j++) {
+		    if (bitmap_id_list[j] == bitmap_id) {
+		        swf_tag_shape_detail_t *swf_tag_shape = tag->detail;
+			add_index_long(return_value, i, (long) swf_tag_shape->shape_id);
+			i++;
+			break;
+		    }
+		}
+		free(bitmap_id_list);
+	    }
         }
     }
 }
@@ -1184,6 +1320,7 @@ PHP_METHOD(swfed, replaceMovieClip) {
         if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ssb",
 				  &instance_name, &instance_name_len,
 				  &swf_data, &swf_data_len, &unused_cid_purge) == FAILURE) {
+            // unused_cid_purge ̵뤷ޤ
 	  RETURN_FALSE;
 	}
 	break;
@@ -1191,23 +1328,24 @@ PHP_METHOD(swfed, replaceMovieClip) {
     swf = get_swf_object(getThis() TSRMLS_CC);    
     result = swf_object_replace_movieclip(swf, instance_name,
                                           instance_name_len,
-                                          swf_data, swf_data_len,
-                                          unused_cid_purge);
+                                          swf_data, swf_data_len);
     if (result) {
         RETURN_FALSE;
     }
     RETURN_TRUE;
 }
 
-PHP_METHOD(swfed, swfInfo) {
+PHP_METHOD(swfed, rebuild) {
     swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
-    swf_object_print(swf);
+    if (swf_object_rebuild(swf)) {
+        RETURN_FALSE;
+    }
     RETURN_TRUE;
 }
 
-PHP_METHOD(swfed, rebuild) {
+PHP_METHOD(swfed, purgeUselessContents) {
     swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
-    swf_object_rebuild(swf);
+    swf_object_purge_contents(swf);
     RETURN_TRUE;
 }
 
@@ -1222,6 +1360,35 @@ PHP_METHOD(swfed, setCompressLevel) {
     RETURN_TRUE;
 }
 
+PHP_METHOD(swfed, isShapeTagData) {
+    swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
+    char *data = NULL;
+    int data_len = 0;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
+                              "s", &data, &data_len) == FAILURE) {
+        RETURN_FALSE;
+    }
+    if (swf_object_is_shape_tagdata(data, data_len) == 0) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;      
+}
+
+PHP_METHOD(swfed, isBitmapTagData) {
+    swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
+    char *data = NULL;
+    int data_len = 0;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
+                              "s", &data, &data_len) == FAILURE) {
+        RETURN_FALSE;
+    }
+    if (swf_object_is_bitmap_tagdata(data, data_len) == 0) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;      
+}
+
+
 static swf_object_t  *get_swf_object(zval *obj TSRMLS_DC) {
 //    zval *data, **tmp;
     zval **tmp = NULL;
@@ -1245,3 +1412,4 @@ static void free_swfed_resource(zend_rsrc_list_entry *resource TSRMLS_DC)
 //    printf("SWFEditor->destory\n");
     swf_object_close((swf_object_t *) resource->ptr);
 }
+
