@@ -13,14 +13,14 @@
 struct swf_object_; // swf_object.h
 
 typedef struct swf_tag_ {
-    int tag;
+    int code;
     unsigned long length;
     int length_longformat;
     //
     unsigned char *data;
     void *detail;
     //
-    struct swf_tag_ *next;
+    struct swf_tag_ *prev, *next;
 } swf_tag_t;
 
 typedef struct swf_tag_detail_handler_ {
