@@ -21,7 +21,8 @@ extern swf_tag_detail_handler_t *swf_tag_action_detail_handler(void);
 extern void *swf_tag_action_create_detail(void);
 extern int swf_tag_action_input_detail(swf_tag_t *tag,
                                        struct swf_object_ *swf);
-extern int swf_tag_action_identity_detail(swf_tag_t *tag, int id);
+extern int swf_tag_action_get_cid_detail(swf_tag_t *tag);
+extern int swf_tag_action_replace_cid_detail(swf_tag_t *tag, int id);
 extern unsigned char *swf_tag_action_output_detail(swf_tag_t *tag,
                                                    unsigned long *length,
                                                    struct swf_object_ *swf);
@@ -31,5 +32,6 @@ extern void swf_tag_action_print_detail(swf_tag_t *tag,
 extern void swf_tag_action_destroy_detail(swf_tag_t *tag);
 
 extern int swf_tag_action_create_setvaribles(swf_tag_t *tag, y_keyvalue_t *kv);
+extern int swf_tag_action_put_setvaribles(swf_tag_t *tag, y_keyvalue_t *kv);
 
 #endif /* __SWF_TAG_ACTION__H__ */
